<?php

/**
 * All registered action's handlers should be in app\Hooks\Handlers,
 * addAction is similar to add_action and addCustomAction is just a
 * wrapper over add_action which will add a prefix to the hook name
 * using the plugin slug to make it unique in all wordpress plugins,
 * ex: $app->addCustomAction('foo', ['FooHandler', 'handleFoo']) is
 * equivalent to add_action('slug-foo', ['FooHandler', 'handleFoo']).
 */

/**
 * $app
 * @var ExactLinks\Framework\Foundation\Application
 */


$app->addCustomAction('handle_exception', 'ExceptionHandler@handle');
$app->addAction('admin_menu', 'AdminPageHandler@addMenuPage');
$app->addAction('admin_enqueue_scripts', 'AdminPageHandler@loadAssets');
$app->addAction('admin_init', 'AdminPageHandler@adminNotice');
$app->addAction('init', 'RenderHandler@initLoading');

if (php_sapi_name() != 'cli') {
    
    if(defined('EXACTLINKS_VERSION')) {
        $app->addAction('rest_api_init', 'GutenbergApiHandler@registerRoutes');
        $app->addAction('enqueue_block_editor_assets', 'AdminPageHandler@gutenBlocksEditorAssets');
    }

    if (!is_admin() && $_SERVER['REQUEST_METHOD'] == 'GET') {
        $app->addAction('init', 'FrontendHandler@redirectionURL', 1);
    }
    
    if (!is_admin() && get_option('exactlinks_db_active')) {
        $app->addAction('wp_footer', 'FrontendHandler@googleAnaliticsTrack');
        
        if (defined('WC_VERSION')) {
            $app->addAction('woocommerce_checkout_create_order', 'FrontendHandler@maybeCampaignMeta', 10, 2);
            $app->addAction('woocommerce_order_status_processing', 'FrontendHandler@maybeRecordPayment', 10, 2);
        }

        if(defined('EDD_VERSION')) {
            $app->addAction( 'edd_pre_complete_purchase', 'FrontendHandler@eddCampaignMeta', 10, 1);
            $app->addAction( 'edd_complete_purchase', 'FrontendHandler@eddCompletePurchase', 10, 2);
        }

    }
}

